@@ -3,23 +3,23 @@
 #' Download a dataset using the BFS offical API v1. You should choose either the bfs number of the bfs offical url
 #' of a given dataset. You can query particulary variables using the `query` argument.
 #'
-#' @param url_bfs The URL page of a dataset.
 #' @param number_bfs The BFS number of a dataset.
-#' @param language Language of the dataset to be translated if exists.
-#' @param query a list with named values, a json query file or json query string using \code{pxweb::pxweb_query()}.
-#' @param column_name_type column name type as "text" or as "code".
-#' @param variable_value_type variable value type as "text" or as "code".
+#' @param language Language of the dataset to be translated if exists, i.e. "de", "fr", "it" or "en".
+#' @param url_bfs The URL page of a dataset.
+#' @param query A list with named values, a json query file or json query string using \code{pxweb::pxweb_query()}.
+#' @param column_name_type Column name type as "text" or as "code".
+#' @param variable_value_type Variable value type as "text" or as "code".
 #' @param clean_names Clean column names using \code{janitor::clean_names()}.
 #'
 #' @seealso \code{\link{bfs_get_data_comments}}
 #'
 #' @return A tbl_df (a type of data frame; see tibble or
 #' dplyr packages).
-#'
+#' 
 #' @export
-bfs_get_data <- function(url_bfs = NULL, language = "de", number_bfs = NULL, query = "all", column_name_type = "text", variable_value_type = "text", clean_names = FALSE) {
+bfs_get_data <- function(number_bfs = NULL, language = "de", url_bfs = NULL, query = NULL, column_name_type = "text", variable_value_type = "text", clean_names = FALSE) {
   
-  if (missing(language)) stop("must choose a language, either 'de', 'fr', 'it' or 'en'", call. = FALSE)
+  #if (missing(language)) stop("must choose a language, either 'de', 'fr', 'it' or 'en'", call. = FALSE)
   language <- match.arg(arg = language, choices = c("de", "fr", "it", "en"))
   
   if(is.null(number_bfs) & is.null(url_bfs)) { stop("Please fill url_bfs or number_bfs", call. = FALSE) }
@@ -35,9 +35,15 @@ bfs_get_data <- function(url_bfs = NULL, language = "de", number_bfs = NULL, que
   }
   
   pxweb_api_url <- paste0("https://www.pxweb.bfs.admin.ch/api/v1/", language, "/", number_bfs, "/", number_bfs, ".px")
-  df_json <- jsonlite::fromJSON(txt = pxweb_api_url)
   
-  if(any(query == "all")) {
+  # if too many requests HTTP 429
+  df_json <- httr2::request("https://www.pxweb.bfs.admin.ch/api/v1") %>% 
+    httr2::req_url_path_append(paste0(language, "/", number_bfs, "/", number_bfs, ".px")) %>% 
+    httr2::req_retry(max_tries = 2, max_seconds = 5) %>%
+    httr2::req_perform() %>% 
+    httr2::resp_body_json(simplifyVector = TRUE)
+  
+  if(any(query == "all" || is.null(query))) {
     variables <- df_json$variables$code
     values <- df_json$variables$values
     df <- rbind(rep("*", length(values)))
